// -----------------------------------------------------------------------------
//
// Author(s):     Peter Schraut (www.console-dev.de)
// Last Modified: 7th January 2007
//
// Purpose:       Demo shows how to use the Pad-System to move an object around
//                using fixed-point math.
//
//
// -----------------------------------------------------------------------------
#include <hel2.h>
#include "ResourceData.h"

// -----------------------------------------------------------------------------
// Structures
// -----------------------------------------------------------------------------
typedef struct _THeart
{
   THandle hObj;  // Object handle returned by hel_ObjCreate
   sfp32 X;       // X coordinate in fixed-point 16:16 format (see docs)
   sfp32 Y;       // Y coordinate in fixed-point 16:16 format (see docs)
} THeart, *PHeart;

// -----------------------------------------------------------------------------
// Variables
// -----------------------------------------------------------------------------
u8 ATTR_EWRAM g_ObjSystemBuffer[HEL_SUBSYSTEM_OBJ_REQUIREDMEMORY];
THeart ATTR_EWRAM g_Heart;

// -----------------------------------------------------------------------------
// Prototypes
// -----------------------------------------------------------------------------
void VBLInterruptHandler(void);
void CreateHeart(void);


// -----------------------------------------------------------------------------
// Program Entry Point
// -----------------------------------------------------------------------------
int main(void)
{
    ham_Init();
    hel_SysSetPrefetch(TRUE);
    
    // Set background mode and init text
    ham_SetBgMode(0);
    ham_InitText(0);

    // Init Object-System
    hel_ObjInit(g_ObjSystemBuffer);

    // Initialize the Pad-System
    hel_PadInit();
    
    // Create heart object
    CreateHeart();
    
    // Start VBL Interrupt
    hel_IntrStartHandler(INT_TYPE_VBL, (void*)VBLInterruptHandler);
    
    // display instruction text
    ham_DrawText(1, 1,"Use pad to move heart");
    ham_DrawText(1, 2,"Hold -A- for speed boost");
    
    
    for(;;)
    {
         // Call hel_PadCapture once per frame
         // to capture the current pad-state
         hel_PadCapture();

         // display heart position
         ham_DrawText(1, 5,"X: %3d, hel_ObjGetX: %3d", FIXED_TOINT(g_Heart.X), hel_ObjGetX(g_Heart.hObj));
         ham_DrawText(1, 6,"Y: %3d, hel_ObjGetY: %3d", FIXED_TOINT(g_Heart.Y), hel_ObjGetY(g_Heart.hObj));

         // velocity of heart
         TPointSfp32 Velocity={0,0};
         
         if(hel_PadQuery()->Held.Left)
           Velocity.X -= FIXED_FROMFLOAT(1.5f);

         if(hel_PadQuery()->Held.Right)
           Velocity.X += FIXED_FROMFLOAT(1.5f);
           
         if(hel_PadQuery()->Held.Up)
           Velocity.Y -= FIXED_FROMFLOAT(1.5f);
           
         if(hel_PadQuery()->Held.Down)
           Velocity.Y += FIXED_FROMFLOAT(1.5f);
         
         
         // if -A- is held, double speed
         if(hel_PadQuery()->Held.A)
         {
           Velocity.X = FIXED_MUL(Velocity.X, FIXED_FROMFLOAT(2.5f));
           Velocity.Y = FIXED_MUL(Velocity.Y, FIXED_FROMFLOAT(2.5f));
         }
        
         // apply velocity to heart position
         g_Heart.X += Velocity.X;
         g_Heart.Y += Velocity.Y;
         
         // set object position
         hel_ObjSetXY(g_Heart.hObj, FIXED_TOINT(g_Heart.X), FIXED_TOINT(g_Heart.Y));
         
         // Wait for Vertical Blank
         hel_SwiVBlankIntrWait();
    }

    return 0;
}


// -----------------------------------------------------------------------------
// Create heart object
// -----------------------------------------------------------------------------
void CreateHeart(void)
{
   // load palette
   hel_PalObjLoad256(ResData16(RES_OBJECT_PAL));
   
   // create heart
   g_Heart.hObj = hel_ObjCreate256
                  (
                     ResData(RES_HEART_RAW),
                     OBJ_SHAPE_SQUARE,
                     1,
                     OBJ_MODE_NORMAL,
                     241,
                     161
                  );

   // move heart behind text background
   hel_ObjSetPrio(g_Heart.hObj, 1);
   
   // set initial position
   g_Heart.X = FIXED_FROMINT(53);
   g_Heart.Y = FIXED_FROMINT(40);
}


// -----------------------------------------------------------------------------
// VBL Interupt Handler
// -----------------------------------------------------------------------------
void VBLInterruptHandler(void)
{
    // transmit attributes to hardware
    hel_ObjTransmit();
    
    // acknowledge interrupt
    hel_IntrAcknowledge(INT_TYPE_VBL);
}

